@@ -20,6 +20,7 @@ import sys
 import time
 
 from starcluster import node
+from starcluster import utils
 from starcluster import cluster
 from starcluster import completion
 from starcluster.logger import log
@@ -171,11 +172,28 @@ class CmdBase(completion.CmdComplete):
             parser.error("option %s must be a positive integer" % opt_str)
         setattr(parser.values, option.dest, value)
 
+    def _iso_timestamp(self, option, opt_str, value, parser):
+        if not utils.is_iso_time(value):
+            parser.error("option %s must be an iso8601 formatted timestamp" %
+                         opt_str)
+        setattr(parser.values, option.dest, value)
+
+    def _file_exists(self, option, opt_str, value, parser):
+        path = os.path.abspath(os.path.expanduser(os.path.expandvars(value)))
+        if not os.path.exists(path):
+            parser.error("(%s) file does not exist: %s" % (opt_str, path))
+        if not os.path.isfile(path):
+            parser.error("(%s) path is not a file: %s" % (opt_str, path))
+        setattr(parser.values, option.dest, path)
+
     def _build_dict(self, option, opt_str, value, parser):
         tagdict = getattr(parser.values, option.dest)
         tags = value.split(',')
         for tag in tags:
             tagparts = tag.split('=')
+            if len(tagparts) != 2:
+                parser.error("invalid tag: '%s' (correct example: key=value)" %
+                             tag)
             key = tagparts[0]
             if not key:
                 continue
@@ -191,3 +209,11 @@ class CmdBase(completion.CmdComplete):
                 tagstore = value
             tagdict[key] = tagstore
         setattr(parser.values, option.dest, tagdict)
+
+    def _get_duplicate(self, lst):
+        d = {}
+        for item in lst:
+            if item in d:
+                return item
+            else:
+                d[item] = 0
