@@ -55,7 +55,7 @@ def create_sc_config_dirs():
     __makedirs(STARCLUSTER_LOG_DIR)
 
 
-VERSION = "0.94.3"
+VERSION = "0.95"
 PID = os.getpid()
 TMP_DIR = tempfile.gettempdir()
 if os.path.exists("/tmp"):
@@ -81,19 +81,19 @@ AWS_DEBUG_FILE = os.path.join(STARCLUSTER_LOG_DIR, 'aws-debug.log')
 CRASH_FILE = os.path.join(STARCLUSTER_LOG_DIR, 'crash-report-%d.txt' % PID)
 
 # StarCluster BASE AMIs (us-east-1)
-BASE_AMI_32 = "ami-7c5c3915"
-BASE_AMI_64 = "ami-765b3e1f"
-BASE_AMI_HVM = "ami-52a0c53b"
+BASE_AMI_32 = "ami-9bf9c9f2"
+BASE_AMI_64 = "ami-3393a45a"
+BASE_AMI_HVM = "ami-6b211202"
 
-SECURITY_GROUP_PREFIX = "@sc"
-SECURITY_GROUP_TEMPLATE = '-'.join([SECURITY_GROUP_PREFIX, "%s"])
+SECURITY_GROUP_PREFIX = "@sc-"
+SECURITY_GROUP_TEMPLATE = SECURITY_GROUP_PREFIX + "%s"
 VOLUME_GROUP_NAME = "volumecreator"
-VOLUME_GROUP = SECURITY_GROUP_TEMPLATE % VOLUME_GROUP_NAME
+VOLUME_GROUP = SECURITY_GROUP_PREFIX + VOLUME_GROUP_NAME
 
 # Cluster group tag keys
-VERSION_TAG = '-'.join([SECURITY_GROUP_PREFIX, 'version'])
-CORE_TAG = '-'.join([SECURITY_GROUP_PREFIX, 'core'])
-USER_TAG = '-'.join([SECURITY_GROUP_PREFIX, 'user'])
+VERSION_TAG = SECURITY_GROUP_PREFIX + 'version'
+CORE_TAG = SECURITY_GROUP_PREFIX + 'core'
+USER_TAG = SECURITY_GROUP_PREFIX + 'user'
 
 # Internal StarCluster userdata filenames
 UD_PLUGINS_FNAME = "_sc_plugins.txt"
@@ -118,23 +118,35 @@ INSTANCE_TYPES = {
     'm2.xlarge': ['x86_64'],
     'm2.2xlarge': ['x86_64'],
     'm2.4xlarge': ['x86_64'],
+    'm3.medium': ['x86_64'],
+    'm3.large': ['x86_64'],
     'm3.xlarge': ['x86_64'],
     'm3.2xlarge': ['x86_64'],
     'cc1.4xlarge': ['x86_64'],
     'cc2.8xlarge': ['x86_64'],
     'cg1.4xlarge': ['x86_64'],
+    'g2.2xlarge': ['x86_64'],
     'cr1.8xlarge': ['x86_64'],
     'hi1.4xlarge': ['x86_64'],
     'hs1.8xlarge': ['x86_64'],
+    'c3.large': ['x86_64'],
+    'c3.xlarge': ['x86_64'],
+    'c3.2xlarge': ['x86_64'],
+    'c3.4xlarge': ['x86_64'],
+    'c3.8xlarge': ['x86_64'],
+    'i2.xlarge': ['x86_64'],
+    'i2.2xlarge': ['x86_64'],
+    'i2.4xlarge': ['x86_64'],
+    'i2.8xlarge': ['x86_64'],
 }
 
 MICRO_INSTANCE_TYPES = ['t1.micro']
 
-SEC_GEN_TYPES = ['m3.xlarge', 'm3.2xlarge']
+SEC_GEN_TYPES = ['m3.medium', 'm3.large', 'm3.xlarge', 'm3.2xlarge']
 
 CLUSTER_COMPUTE_TYPES = ['cc1.4xlarge', 'cc2.8xlarge']
 
-CLUSTER_GPU_TYPES = ['cg1.4xlarge']
+CLUSTER_GPU_TYPES = ['g2.2xlarge', 'cg1.4xlarge']
 
 CLUSTER_HIMEM_TYPES = ['cr1.8xlarge']
 
@@ -142,13 +154,32 @@ HI_IO_TYPES = ['hi1.4xlarge']
 
 HI_STORAGE_TYPES = ['hs1.8xlarge']
 
-CLUSTER_TYPES = CLUSTER_COMPUTE_TYPES + CLUSTER_GPU_TYPES + CLUSTER_HIMEM_TYPES
+M3_COMPUTE_TYPES = ['c3.large', 'c3.xlarge', 'c3.2xlarge', 'c3.4xlarge',
+                    'c3.8xlarge']
 
-HVM_TYPES = CLUSTER_TYPES + HI_IO_TYPES + HI_STORAGE_TYPES + SEC_GEN_TYPES
+I2_STORAGE_TYPES = ['i2.xlarge', 'i2.2xlarge', 'i2.4xlarge', 'i2.8xlarge']
 
-PLACEMENT_GROUP_TYPES = CLUSTER_TYPES + HI_IO_TYPES + HI_STORAGE_TYPES
+HVM_ONLY_TYPES = (CLUSTER_COMPUTE_TYPES + CLUSTER_GPU_TYPES +
+                  CLUSTER_HIMEM_TYPES + I2_STORAGE_TYPES)
 
-CLUSTER_REGIONS = ['us-east-1', 'us-west-2', 'eu-west-1']
+HVM_TYPES = (HVM_ONLY_TYPES + HI_IO_TYPES + HI_STORAGE_TYPES + SEC_GEN_TYPES +
+             M3_COMPUTE_TYPES)
+
+EBS_ONLY_TYPES = MICRO_INSTANCE_TYPES
+
+# Always make sure these match instances listed here:
+# http://docs.aws.amazon.com/AWSEC2/latest/UserGuide/placement-groups.html
+# StarCluster additionally adds cc1.4xlarge to the list - EC2 is slowly
+# migrating folks away from this type in favor of cc2.8xlarge but the type
+# still works for some older accounts.
+PLACEMENT_GROUP_TYPES = (M3_COMPUTE_TYPES + HVM_ONLY_TYPES + HI_IO_TYPES +
+                         HI_STORAGE_TYPES)
+
+# Only add a region to this list after testing that you can create and delete a
+# placement group there.
+PLACEMENT_GROUP_REGIONS = ['us-east-1', 'us-west-2', 'eu-west-1',
+                           'ap-northeast-1', 'ap-southeast-1',
+                           'ap-southeast-2']
 
 PROTOCOLS = ['tcp', 'udp', 'icmp']
 
@@ -227,6 +258,8 @@ CLUSTER_SETTINGS = {
     'cluster_size': (int, True, None, None, None),
     'cluster_user': (str, False, 'sgeadmin', None, None),
     'cluster_shell': (str, False, 'bash', AVAILABLE_SHELLS.keys(), None),
+    'subnet_id': (str, False, None, None, None),
+    'public_ips': (str, False, True, None, None),
     'master_image_id': (str, False, None, None, None),
     'master_instance_type': (str, False, None, INSTANCE_TYPES.keys(), None),
     'node_image_id': (str, True, None, None, None),
@@ -241,4 +274,5 @@ CLUSTER_SETTINGS = {
     'disable_queue': (bool, False, False, None, None),
     'force_spot_master': (bool, False, False, None, None),
     'disable_cloudinit': (bool, False, False, None, None),
+    'dns_prefix': (bool, False, False, None, None),
 }
