@@ -36,33 +36,34 @@ import urlparse
 from datetime import datetime
 
 import iptools
+import iso8601
 import decorator
 
 from starcluster import spinner
 from starcluster import exception
 from starcluster.logger import log
 
-try:
-    import IPython
-    if IPython.__version__ < '0.11':
-        from IPython.Shell import IPShellEmbed
-        ipy_shell = IPShellEmbed(argv=[])
-    else:
-        from IPython import embed
-        ipy_shell = lambda local_ns=None: embed(user_ns=local_ns)
-except ImportError, e:
 
-    def ipy_shell(local_ns=None):
+def ipy_shell(local_ns=None):
+    try:
+        import IPython
+        if IPython.__version__ < '0.11':
+            from IPython.Shell import IPShellEmbed
+            return IPShellEmbed(argv=[])(local_ns)
+        else:
+            from IPython import embed
+            return embed(user_ns=local_ns)
+    except ImportError as e:
         log.error("Unable to load IPython:\n\n%s\n" % e)
         log.error("Please check that IPython is installed and working.")
         log.error("If not, you can install it via: easy_install ipython")
 
-try:
-    import pudb
-    set_trace = pudb.set_trace
-except ImportError:
 
-    def set_trace():
+def set_trace():
+    try:
+        import pudb
+        return pudb.set_trace()
+    except ImportError:
         log.error("Unable to load PuDB")
         log.error("Please check that PuDB is installed and working.")
         log.error("If not, you can install it via: easy_install pudb")
@@ -179,6 +180,20 @@ def is_valid_image_name(image_name):
         return False
 
 
+def is_valid_hostname(hostname):
+    """From StackOverflow on 2013-10-04:
+
+    http://stackoverflow.com
+    /questions/2532053/validate-a-hostname-string#answer-2532344
+    """
+    if len(hostname) > 255:
+        return False
+    if hostname[-1] == ".":
+        hostname = hostname[:-1]  # strip exactly one dot from the right
+    allowed = re.compile("(?!-)[A-Z\d-]{1,63}(?<!-)$", re.IGNORECASE)
+    return all(allowed.match(x) for x in hostname.split("."))
+
+
 def make_one_liner(script):
     """
     Returns command to execute python script as a one-line python program
@@ -224,7 +239,7 @@ def is_iso_time(iso):
     try:
         iso_to_datetime_tuple(iso)
         return True
-    except ValueError:
+    except iso8601.ParseError:
         return False
 
 
@@ -232,20 +247,26 @@ def iso_to_datetime_tuple(iso):
     """
     Converts an iso time string to a datetime tuple
     """
-    #remove timezone
-    iso = iso.split('.')[0]
-    try:
-        return datetime.strptime(iso, "%Y-%m-%dT%H:%M:%S")
-    except AttributeError:
-        # python2.4 datetime module doesnt have strptime
-        return datetime(*time.strptime(iso, "%Y-%m-%dT%H:%M:%S")[:6])
+    return iso8601.parse_date(iso)
+
+
+def get_utc_now(iso=False):
+    """
+    Returns datetime.utcnow with UTC timezone info
+    """
+    now = datetime.utcnow().replace(tzinfo=iso8601.iso8601.UTC)
+    if iso:
+        return datetime_tuple_to_iso(now)
+    else:
+        return now
 
 
 def datetime_tuple_to_iso(tup):
     """
-    Converts a datetime tuple to iso time string
+    Converts a datetime tuple to a UTC iso time string
     """
-    iso = datetime.strftime(tup, "%Y-%m-%dT%H:%M:%S")
+    iso = datetime.strftime(tup.astimezone(iso8601.iso8601.UTC),
+                            "%Y-%m-%dT%H:%M:%S.%fZ")
     return iso
 
 
